IF OBJECT_ID('TR__ApplicationGroups__CircularParentCheck') IS NOT NULL
DROP TRIGGER [TR__ApplicationGroups__CircularParentCheck]
GO

CREATE TRIGGER [TR__ApplicationGroups__CircularParentCheck]
ON [ApplicationGroups] AFTER UPDATE, INSERT AS
BEGIN

	SET NOCOUNT ON

	DECLARE @ApplicationGroups TABLE(
		[Base_ApplicationGroup_Id] INT NOT NULL, 

		[Resolution_Depth] INT NOT NULL,
		[ApplicationGroup_Id] INT NOT NULL, 
		[Parent_ApplicationGroup_Id] INT NULL
	)

	DECLARE @depth INT, @row_count INT
	    SET @depth = 1
	    SET @row_count = 1 
	 
	INSERT INTO @ApplicationGroups 
	  ([Base_ApplicationGroup_Id], [Resolution_Depth], [ApplicationGroup_Id], [Parent_ApplicationGroup_Id])
	SELECT 
	   [ApplicationGroup_Id], 1, [ApplicationGroup_Id], [Parent_ApplicationGroup_Id]
	  FROM [inserted] 
     

	WHILE @row_count > 0
	BEGIN 
		SET @depth = @depth + 1

		-- insert all of the previous depth's parents from the actual table
		INSERT INTO @ApplicationGroups 
		  ([Base_ApplicationGroup_Id], [Resolution_Depth], [ApplicationGroup_Id], [Parent_ApplicationGroup_Id])
		SELECT
		  [Base_ApplicationGroup_Id], @depth, AG.[Parent_ApplicationGroup_Id], PAG.[Parent_ApplicationGroup_Id]
		FROM @ApplicationGroups AG
	         INNER JOIN [ApplicationGroups] PAG
		             ON PAG.[ApplicationGroup_Id] = AG.[Parent_ApplicationGroup_Id]
		WHERE AG.[Resolution_Depth] = @depth - 1

		SET @row_count = @@ROWCOUNT
	
		-- if an application group is an ancestor of itself then shed tears
		IF EXISTS( SELECT * 
		             FROM [inserted] I
		                  INNER JOIN @ApplicationGroups AG
		                          ON AG.[Resolution_Depth] > 1
		                         AND I.[ApplicationGroup_Id] = AG.[Base_ApplicationGroup_Id]
		                         AND I.[ApplicationGroup_Id] = AG.[Parent_ApplicationGroup_Id] )

		BEGIN 
				RAISERROR('The specified parent application group ID will cause a cycle at depth %i, which is not supported.',16,2, @depth)
				ROLLBACK
				RETURN 
		END 

		-- sanity check
		IF @depth > 1000 BEGIN
				RAISERROR('Depth exceeded 1000.',16,1) 
				ROLLBACK
				RETURN 
		END 
	END 


END
GO