IF OBJECT_ID('[Releases_Extended]') IS NOT NULL DROP VIEW [Releases_Extended]
GO

CREATE VIEW [Releases_Extended] AS

SELECT R.[Release_Id]
      ,R.[Application_Id]
	  ,R.[Release_Number]
      ,R.[ReleaseStatus_Name]
      ,R.[Notes_Text]
	  ,R.[CreatedBy_User_Name]
	  ,R.[CreatedOn_Date]
	  ,R.[ModifiedBy_User_Name]
	  ,R.[ModifiedOn_Date]
	  ,R.[CancelledBy_User_Name]
	  ,R.[CancelledReason_Text]
	  ,R.[CancelledOn_Date]
      ,[Target_Date] = RT_LATEST_END.[End_Date]
      ,[Earliest_Target_Start_Date] = RT_EARLIEST_START.[Start_Date]
      ,[Latest_Target_End_Date] = RT_LATEST_END.[End_Date]
      ,[Release_Name] = COALESCE(R.[Release_Name], R.[Release_Number])
      ,R.[Release_Sequence]
	  ,R.[Pipeline_Name]
	  ,A.[Application_Name]
	  ,R.[Furthest_Build_Number]
	  ,B_Furthest.[Furthest_PipelineStage_Name]
      ,[Furthest_Build_Id] = B_Furthest.[Build_Id]
	  ,[Furthest_Build_CreatedOn_Date] = B_Furthest.[CreatedOn_Date]
      ,[Latest_Build_Id] = B_Latest.[Build_Id]
	  ,[Latest_Build_Number] = B_Latest.[Build_Number]
	  ,[Latest_Build_CreatedOn_Date] = B_Latest.[CreatedOn_Date]
	  ,[Latest_Promotion_Id] = BP_Latest.[Promotion_Id]
	  ,[Latest_Promoted_Date] = BP_Latest.[Promoted_Date]
	  ,[Latest_PromotionStatus_Name] = BP_Latest.[PromotionStatus_Name]
	  ,[Latest_Promotion_PipelineStage_Name] = BP_Latest.[PipelineStage_Name]
      ,[Latest_Execution_Id] = E_Latest.[Execution_Id]
      ,[Latest_Execution_Start_Date] = E_Latest.[Start_Date]
      ,[Latest_Execution_RunState_Code] = E_Latest.[ExecutionRunState_Code]
      ,[Latest_Execution_Status_Code] = E_Latest.[ExecutionStatus_Code]
      ,[Latest_Execution_Environment_Id] = BE_Latest_Env.[Environment_Id]
      ,R.[ReleaseTemplate_Name]

  FROM [Releases] R
	   INNER JOIN [Applications] A
               ON R.[Application_Id] = A.[Application_Id]
	    LEFT JOIN [Builds] B_Furthest
		       ON B_Furthest.[Release_Id] = R.[Release_Id]
		      AND B_Furthest.[Build_Number] = R.[Furthest_Build_Number]
		LEFT JOIN (SELECT *, [Row] = ROW_NUMBER() OVER (PARTITION BY [Release_Id] ORDER BY [Build_Id] DESC)
		             FROM [Builds]) B_Latest
			   ON B_Latest.[Release_Id] = R.[Release_Id]
			  AND B_Latest.[Row] = 1
		LEFT JOIN (SELECT _BP.*,
		                  _B.[Release_Id],
						  [Row] = ROW_NUMBER() OVER (PARTITION BY _B.[Release_Id] ORDER BY _BP.[Promotion_Id] DESC)
		             FROM [BuildPromotions] _BP
					      INNER JOIN [Builds] _B
						          ON _B.[Build_Id] = _BP.[Build_Id]) BP_Latest
			   ON BP_Latest.[Release_Id] = R.[Release_Id]
			  AND BP_Latest.[Row] = 1
        OUTER APPLY(
            SELECT TOP 1 BE.* FROM [PipelineStageTargetExecutions] BE
             INNER JOIN [BuildPromotions] BP ON BP.[Promotion_Id] = BE.[Promotion_Id]
             INNER JOIN [Builds] B ON B.[Build_Id] = BP.[Build_Id]
             INNER JOIN [Executions] E ON E.[Execution_Id] = BE.[Execution_Id]
                  WHERE B.[Release_Id] = R.[Release_Id]
               ORDER BY E.[Execution_Id] DESC
        ) BE_Latest
        OUTER APPLY(
            SELECT TOP 1 * FROM [ReleaseTargetDates] RTD WHERE RTD.[Release_Id] = R.[Release_Id] ORDER BY RTD.[Start_Date] ASC
        ) RT_EARLIEST_START
        OUTER APPLY(
            SELECT TOP 1 * FROM [ReleaseTargetDates] RTD WHERE RTD.[Release_Id] = R.[Release_Id] ORDER BY RTD.[End_Date] DESC
        ) RT_LATEST_END
        
        LEFT JOIN [Executions] E_Latest 
               ON E_Latest.[Execution_Id] = BE_Latest.[Execution_Id]

	    LEFT JOIN [Environments] BE_Latest_Env
		       ON BE_Latest_Env.[Environment_Name] = BE_Latest.[Environment_Name]

GO