EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Rafts_CreateOrUpdateRaftItem',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Creates an item in a database raft.'
GO

IF OBJECT_ID('[Rafts_CreateOrUpdateRaftItem]') IS NOT NULL DROP PROCEDURE [Rafts_CreateOrUpdateRaftItem]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Rafts_CreateOrUpdateRaftItem]
(
	@Raft_Id INT,
	@RaftItemType_Code INT,
	@RaftItem_Name VARCHAR(255),
	@ModifiedOn_Date DATETIMEOFFSET,
	@ModifiedBy_User_Name USERNAME,
	@Content_Bytes VARBINARY(MAX),
	@Application_Id INT,
	@RaftItem_Id INT = NULL OUT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	SET @RaftItem_Id = (SELECT [RaftItem_Id]
	                      FROM [RaftItems]
						 WHERE [Raft_Id] = @Raft_Id
						   AND ([Application_Id] = @Application_Id OR ([Application_Id] IS NULL AND @Application_Id IS NULL))
						   AND [RaftItemType_Code] = @RaftItemType_Code
						   AND [RaftItem_Name] = @RaftItem_Name)

	DECLARE @RaftItemVersion_Number INT

	IF @RaftItem_Id IS NULL BEGIN
		INSERT INTO [RaftItems]
		(
			[Raft_Id],
			[RaftItemType_Code],
			[RaftItem_Name],
			[Active_Indicator],
			[Application_Id]
		)
		VALUES
		(
			@Raft_Id,
			@RaftItemType_Code,
			@RaftItem_Name,
			'Y',
			@Application_Id
		)
	
		SET @RaftItem_Id = SCOPE_IDENTITY()
		SET @RaftItemVersion_Number = 1
	END ELSE BEGIN
		UPDATE [RaftItems]
		   SET [Active_Indicator] = 'Y'
		 WHERE [RaftItem_Id] = @RaftItem_Id

		 SET @RaftItemVersion_Number = COALESCE((SELECT MAX([RaftItemVersion_Number]) FROM [RaftItemVersions] WHERE [RaftItem_Id] = @RaftItem_Id), 0) + 1
	END

	INSERT INTO [RaftItemVersions]
	(
		[RaftItem_Id],
		[RaftItemVersion_Number],
		[ModifiedOn_Date],
		[ModifiedBy_User_Name],
		[Content_Bytes]
	)
	VALUES
	(
		@RaftItem_Id,
		@RaftItemVersion_Number,
		@ModifiedOn_Date,
		@ModifiedBy_User_Name,
		@Content_Bytes
	)

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [Rafts_CreateOrUpdateRaftItem] TO [BuildMasterUser_Role]
