EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Servers_UpdateServerStatus',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ NULL,
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Updates a server''s status.'
GO

IF OBJECT_ID('[Servers_UpdateServerStatus]') IS NOT NULL DROP PROCEDURE [Servers_UpdateServerStatus]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Servers_UpdateServerStatus]
(
	@Server_Id INT,
	@ServerStatus_Code CHAR(5),
	@Agent_Version VARCHAR(20) = NULL
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

        DECLARE @Old_ServerStatus_Code CHAR(5)
        
        SELECT @Old_ServerStatus_Code = [ServerStatus_Code]
	      FROM [Servers]
	     WHERE [Server_Id] = @Server_Id

	    UPDATE [Servers]
	       SET [ServerStatus_Code] = COALESCE(@ServerStatus_Code, [ServerStatus_Code]),
		       [Agent_Version] = COALESCE(@Agent_Version, [Agent_Version])
	     WHERE [Server_Id] = @Server_Id
	     
	     -- raise server error event if the status changes to ERROR from something else
	     IF (@ServerStatus_Code = 'ERROR' AND @Old_ServerStatus_Code <> 'ERROR')
	        EXEC [Events_RaiseEvent] 'SRVERR', NULL, @Server_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Servers_UpdateServerStatus] TO [BuildMasterUser_Role]